package com.jkcredit.info.query.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.common.constant.CommonConstant;
import com.jkcredit.common.model.CommonResponseObject;
import com.jkcredit.info.query.enums.CatalogEnum;
import com.jkcredit.info.query.service.InformationQueryService;
import com.jkcredit.info.query.task.AsyncQuery;
import com.jkcredit.info.query.util.*;
import lombok.extern.slf4j.Slf4j;
import okhttp3.FormBody;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class InformationQueryServiceImpl implements InformationQueryService {
    @Value("${server.port}")
    private String serverPort;
    @Value("${supplier.mainPageRequestUrl}")
    private String mainPageRequestUrl;
    @Value("${supplier.catalogUrl}")
    private String catalogUrl;
    @Value("${supplier.queryResultUrl}")
    private String queryResultUrl;
    @Value("${supplier.hostUrl}")
    private String hostUrl;
    @Autowired
    AsyncQuery asyncQuery;

    @Override
    public CommonResponseObject query(String keyWord, String type) {
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        log.info("keyWord:{}, type:{}, requestId:{}", keyWord, type, requestId);
        long startTime = System.currentTimeMillis();
        String cookie = getCookie();
        if (StringUtils.isBlank(cookie)) {
            CommonResponseObject result = CommonResponseObject.failed("查询失败",
                    CommonConstant.NETWORK_DISABLED, getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
        Map<String, JSONObject> catalogIdMap = CatalogConcurrentHashMapUtil.getInstance();
        if (catalogIdMap.size() == 0) {
            // 查询节点id
            FormBody formBody = new FormBody.Builder()
                    .add("pinyin", "false")
                    .add("keyWord", keyWord)
                    .build();
            String catalogContent = HttpUtil.doPost(catalogUrl, formBody, cookie);
            catalogIdMap = getCatalogIds(catalogContent);
            CatalogConcurrentHashMapUtil.getInstance().putAll(catalogIdMap);
        }

        String id = "";
        String queryId = "";
        for (Map.Entry<String, JSONObject> entry : catalogIdMap.entrySet()) {
            String catalogName = entry.getKey();
            if (type.equals(catalogName)) {
                JSONObject catalogIdObject = entry.getValue();
                queryId = catalogIdObject.getString("queryId");
                id = catalogIdObject.getString("id");
            } else {
                continue;
            }
        }
        String encodeKeyWord = "";
        try {
            encodeKeyWord = URLEncoder.encode(keyWord, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StringBuffer stringBuffer = new StringBuffer(queryResultUrl);
        stringBuffer.append("?pinyin=false");
        stringBuffer.append("&keyWord=").append(encodeKeyWord);
        stringBuffer.append("&reportQuery.id=").append(queryId);
        stringBuffer.append("&resource.id=").append(id);
        String resultContent = HttpUtil.doGet(stringBuffer.toString(), cookie);
        List<Map<String, String>> detailInfos = getDetailInfo(resultContent);

        if (detailInfos.size() != 0) {
            CommonResponseObject result = CommonResponseObject.success(detailInfos, getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        } else {
            CommonResponseObject result = CommonResponseObject.noResult(getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
    }

    private String getCookie() {
        Map<String, Long> cookieTaskMap = CookieTaskConcurrentHashMapUtil.getInstance();
        if (cookieTaskMap.size() == 0) {
            String cookie = CookieUtil.getNewCookie();
            if (StringUtils.isNotBlank(cookie)) {
                log.info("firstGetCookie:{}", cookie);
                CookieTaskConcurrentHashMapUtil.getInstance().put("cookie", System.currentTimeMillis());
                log.info("InformationQueryServiceImpl.getCookie:{}", cookie);
            }
            return cookie;
        } else {
            long cookieTime = cookieTaskMap.get("cookie");
            if (System.currentTimeMillis() - cookieTime >= 1620000) {
                String cookie = CookieUtil.getNewCookie();
                if (StringUtils.isNotBlank(cookie)) {
                    log.info("reloadCookie:{}", cookie);
                    CookieTaskConcurrentHashMapUtil.getInstance().put("cookie", System.currentTimeMillis());
                    log.info("InformationQueryServiceImpl.getCookie:{}", cookie);
                }
                return cookie;
            } else {
                log.info("usefulCookie:{}", CookieUtil.getAuthenticationCookie());
                return CookieUtil.getAuthenticationCookie();
            }
        }
    }

    @Override
    public CommonResponseObject queryList(String keyWord, String type) {
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        log.info("keyWord:{}, type:{}, requestId:{}", keyWord, type, requestId);
        long startTime = System.currentTimeMillis();
        String cookie = getCookie();
        if (StringUtils.isBlank(cookie)) {
            CommonResponseObject result = CommonResponseObject.failed("查询失败",
                    CommonConstant.NETWORK_DISABLED, getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
        QueryCountUtil.addQueryInfoListCount();
        Map<String, JSONObject> catalogIdMap = CatalogConcurrentHashMapUtil.getInstance();
        if (catalogIdMap.size() == 0) {
            // 查询节点id
            FormBody formBody = new FormBody.Builder()
                    .add("pinyin", "false")
                    .add("keyWord", keyWord)
                    .build();
            String catalogContent = HttpUtil.doPost(catalogUrl, formBody, cookie);
            catalogIdMap = getCatalogIds(catalogContent);
            CatalogConcurrentHashMapUtil.getInstance().putAll(catalogIdMap);
        }

        String id = "";
        String queryId = "";
        for (Map.Entry<String, JSONObject> entry : catalogIdMap.entrySet()) {
            String catalogName = entry.getKey();
            if (type.equals(catalogName)) {
                JSONObject catalogIdObject = entry.getValue();
                queryId = catalogIdObject.getString("queryId");
                id = catalogIdObject.getString("id");
            } else {
                continue;
            }
        }
        String encodeKeyWord = "";
        try {
            encodeKeyWord = URLEncoder.encode(keyWord, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        StringBuffer stringBuffer = new StringBuffer(queryResultUrl);
        stringBuffer.append("?pinyin=false");
        stringBuffer.append("&keyWord=").append(encodeKeyWord);
        stringBuffer.append("&reportQuery.id=").append(queryId);
        stringBuffer.append("&resource.id=").append(id);
        String resultContent = HttpUtil.doGet(stringBuffer.toString(), cookie);
        List<Map<String, String>> listInfo = getListInfo(type, resultContent);

        if (listInfo.size() != 0) {
            CommonResponseObject result = CommonResponseObject.success(listInfo, getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        } else {
            CommonResponseObject result = CommonResponseObject.noResult(getLocalHost(), requestId);
            log.info("keyWord:{}, type:{}, requestId:{}, result:{}, costTime:{}"
                    , keyWord, type, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
    }

    @Override
    public CommonResponseObject queryDetail(String path) {
        long startTime = System.currentTimeMillis();
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        String cookie = getCookie();
        if (StringUtils.isBlank(cookie)) {
            CommonResponseObject result = CommonResponseObject.failed("查询失败",
                    CommonConstant.NETWORK_DISABLED, getLocalHost(), requestId);
            log.info("path:{}, requestId:{}, result:{}, costTime:{}"
                    , path, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
        log.info("path:{}, requestId:{}", path, requestId);
        if (!path.contains("../")) {
            CommonResponseObject result = CommonResponseObject.noResult(getLocalHost(), requestId);
            log.info("path:{}, requestId:{}, result:{}, costTime:{}"
                    , path, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }

        QueryCountUtil.addQueryDetailCount();
        List<Map<String, String>> results = getDetail(path, cookie);
        if (results.size() != 0) {
            CommonResponseObject result = CommonResponseObject.success(results, getLocalHost(), requestId);
            log.info("path:{}, requestId:{}, result:{}, costTime:{}"
                    , path, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        } else {
            CommonResponseObject result = CommonResponseObject.noResult(getLocalHost(), requestId);
            log.info("path:{}, requestId:{}, result:{}, costTime:{}"
                    , path, requestId, JSON.toJSONString(result), (System.currentTimeMillis() - startTime));
            return result;
        }
    }

    private List<Map<String, String>> getListInfo(String type, String content) {
        List<Map<String, String>> results = new ArrayList<>();

        Document doc = Jsoup.parse(content);
        Elements elementList = doc.select("td").select(".column");
        for (Element element : elementList) {
            String imageUrl = "";
            String url = element.select(".detailImage").attr("data-url");
            if (url.contains("/photoSearch/singlePhotoSearch.action")) {
                imageUrl = url.replace("../photoSearch/singlePhotoSearch.action",
                                "http://51.1.0.67:80/info-query/getImage");
            } else if (url.contains("/crjPhotoSearch/singlePhotoSearch.action")) {
                imageUrl = url.replace("../crjPhotoSearch/singlePhotoSearch.action",
                                "http://51.1.0.67:80/info-query/getCrjImage");
            }

            Elements tdList = element.select(".kyfuse-form-value");
            Map<String, String> tdMap = new HashMap<>();
            for (Element td : tdList) {
                String dataName = td.attr("data-name");
                String dataValue = td.attr("data-value");
                tdMap.put(dataName, dataValue);
            }
            results.add(tdMap);
            if (type.equals("stolenVehicleInfo")) {
                tdMap.put("detailPath", element.select("a").eachAttr("url").get(1));
            } else {
                tdMap.put("detailPath", element.select("a").attr("url"));
            }
            tdMap.put("imageUrl", imageUrl);
        }
        return results;
    }

    private List<Map<String, String>> getDetail(String path, String cookie) {
        List<Map<String, String>> results = new ArrayList<>();
        String url = path.replace("../", hostUrl).replace("amp;", "");
        String detailInfoContent = HttpUtil.doGet(url, cookie);
        Document detailDoc = Jsoup.parse(detailInfoContent);
        Elements elements = detailDoc.select(".detail-info-list");

        String imageUrl = "";
        String srcUrl = detailDoc.select(".detail-phone").attr("src");
        if (srcUrl.contains("/photoSearch/singlePhotoSearch.action")) {
            imageUrl = srcUrl.replace("/photoSearch/singlePhotoSearch.action",
                    "http://51.1.0.67:80/info-query/getImage");
        } else if (srcUrl.contains("/crjPhotoSearch/singlePhotoSearch.action")) {
            imageUrl = srcUrl.replace("/crjPhotoSearch/singlePhotoSearch.action",
                    "http://51.1.0.67:80/info-query/getCrjImage");
        }

        Map<String, String> detailMap = new HashMap<>();
        for (int i = 0; i < elements.size(); i++) {
            String dataName = detailDoc.select(".detail-info-list").get(i).select("span").get(1).attr("data-name");
            String dataValue = detailDoc.select(".detail-info-list").get(i).select("span").get(1).text();
            detailMap.put(dataName, dataValue);
            detailMap.put("imageUrl", imageUrl);
        }
        if (detailMap.size() != 0) {
            results.add(detailMap);
        }
        return results;
    }

    // 处理列表多条返回的情况
    private List<Map<String, String>> getDetailInfo(String content) {
        List<Map<String, String>> results = new ArrayList<>();

        Document doc = Jsoup.parse(content);
        // 获取详情请求url
        List<String> urls = doc.select("a").eachAttr("url");
        CountDownLatch countDownLatch = new CountDownLatch(urls.size());
        List<String> detailContents = new ArrayList<>();
        for (String url : urls) {
            url = url.replace("../", hostUrl);
            try {
                Future<String> detailInfo = asyncQuery.detailInfoQuery(url);
                String detailInfoContent = detailInfo.get();
                detailContents.add(detailInfoContent);
            } catch (InterruptedException ie) {
                log.error("getDetailInfo.InterruptedException:", ie);
            } catch (ExecutionException e) {
                log.error("getDetailInfo.ExecutionException:", e);
            } finally {
                countDownLatch.countDown();
            }
        }
        try {
            countDownLatch.await();
        } catch (InterruptedException ie) {
            log.error("getDetailInfo.InterruptedException:", ie);
        }

        for (String detailInfoContent : detailContents) {
            Document detailDoc = Jsoup.parse(detailInfoContent);
            Elements elements = detailDoc.select(".detail-info-list");
            Map<String, String> detailMap = new HashMap<>();
            for (int i = 0; i < elements.size(); i++) {
                String dataName = detailDoc.select(".detail-info-list").get(i).select("span").get(1).attr("data-name");
                String dataValue = detailDoc.select(".detail-info-list").get(i).select("span").get(1).text();
                detailMap.put(dataName, dataValue);
            }
            if (detailMap.size() != 0) {
                results.add(detailMap);
            }
        }
        return results;
    }

    private Map<String, JSONObject> getCatalogIds(String catalogContent) {
        // dom解析
        Document doc = Jsoup.parse(catalogContent);
        Elements elements = doc.select("li[class=sjzyli]");
        Map<String, JSONObject> catalogIdMap = new HashMap<>();
        for (int i = 0; i < elements.size(); i++) {
            Element element = elements.get(i);

            String name = element.select("li").attr("name");
            String id = element.select("li").attr("id");
            String queryId = element.select("li").attr("queryId");
            JSONObject catalogIdObject = new JSONObject();
            catalogIdObject.put("id", id);
            catalogIdObject.put("queryId", queryId);
            if (name.equals(CatalogEnum.BASIC_INFO.getDesc())) {
                // 全国人口基本信息
                catalogIdMap.put(CatalogEnum.BASIC_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.DRIVER_INFO.getDesc())) {
                // 全国驾驶人
                catalogIdMap.put(CatalogEnum.DRIVER_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.STOLEN_VEHICLE_INFO.getDesc())) {
                // 被盗抢车辆
                catalogIdMap.put(CatalogEnum.STOLEN_VEHICLE_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.IMMIGRATION_INFO.getDesc())) {
                // 出入境证件
                catalogIdMap.put(CatalogEnum.IMMIGRATION_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.ILLEGAL_INFO.getDesc())) {
                // 违法犯罪人员
                catalogIdMap.put(CatalogEnum.ILLEGAL_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.ESCAPED_CRIMINAL_INFO.getDesc())) {
                // 全国在逃人员
                catalogIdMap.put(CatalogEnum.ESCAPED_CRIMINAL_INFO.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.MOVEMENT_RECORD.getDesc())) {
                // 出入境记录
                catalogIdMap.put(CatalogEnum.MOVEMENT_RECORD.getValue(), catalogIdObject);
            } else if (name.equals(CatalogEnum.VEHICLE_INFO.getDesc())) {
                // 全国机动车登记
                catalogIdMap.put(CatalogEnum.VEHICLE_INFO.getValue(), catalogIdObject);
            } else {
                continue;
            }
        }
        return catalogIdMap;
    }

    @Override
    public CommonResponseObject loadCookie() {
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        String cookie = CookieUtil.getAuthenticationCookie();
        if (StringUtils.isBlank(cookie)) {
            return CommonResponseObject.failed("查询失败", CommonConstant.NETWORK_DISABLED, getLocalHost(), requestId);
        } else {
            return CommonResponseObject.success(new ArrayList<>(), getLocalHost(), requestId);
        }
    }

    @Override
    public CommonResponseObject getNewCookie() {
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        String cookie = CookieUtil.getNewCookie();
        if (StringUtils.isBlank(cookie)) {
            return CommonResponseObject.failed("查询失败", CommonConstant.NETWORK_DISABLED, getLocalHost(), requestId);
        } else {
            return CommonResponseObject.success(new ArrayList<>(), getLocalHost(), requestId);
        }
    }

    @Override
    public InputStream getImageStream(String url) {
        String cookie = getCookie();
        if (StringUtils.isBlank(cookie)) {
            log.info("loadCookieFailed,url:{}", url);
            return null;
        }

        return HttpUtil.doGetStream(url, cookie);
    }

    private String getLocalHost() {
        InetAddress address = null;
        try {
            address = InetAddress.getLocalHost();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }
        if (address != null) {
            return address.getHostAddress() + ":" + this.serverPort;
        } else {
            return "";
        }
    }
}
